#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Sun Mar 10 22:11:45 2024

@auteur: david ALBERTO
(www.astrolabe-science.fr)
Ce script trace les lignes horaires de cadrans solaires orientés
sud, nord, est, ouest et horizontal.
Quelques arcs durnes sont aussi tracés.
Principaux paramètres ajustables : latitude, dimensions des cadres,
hauteur du style droit.
ATTENTION : après avoir défini ces paramètres,d'autres paramètres devront être
réajustés : taille de la police, décalage vertical de certains tracés...
Le script est adapté pour une latitude entre 40 et 50° environ ;
pour d'autres valeurs, des ajustements sont à prévoir pour certaines lignes horaires.
"""
import matplotlib.pyplot as plt
import numpy as np

latitude = 49.5 # latitude du lieu (degrés)
largeur = 8 # largeur de chaque cadre en cm
hauteur = 7 # hauteur de chaque cadre, en cm
a = 1.4 # hauteur du style droit (cm)
marge = 0.5 # marge autour du document et entre les cadres (cm)
# apparence des lignes :
prop_lignesH = dict(c='orangered', lw=1, alpha=1, solid_capstyle='round')
prop_lignesdemies = dict(c='gray', lw=0.6, alpha=0.6, solid_capstyle='round')
prop_arcsdiurnes = dict(c='indigo', lw=0.4, alpha = 0.6)

nomFichier = 'cadranCube' + str(latitude)
eps = np.radians(23.44)
rad_15 = np.radians(15)

plt.rcParams["font.family"] = "Roboto Condensed" # ou autre police installée
plt.rcParams["font.size"] = 9

lat_rad = np.radians(latitude)
colat = np.pi/2 - lat_rad
heuremin = 12 - np.arccos(-np.tan(lat_rad) * np.tan(eps)) / rad_15
heuremin = round(heuremin)
heuremax = 24-heuremin

fig, axs = plt.subplots(ncols=3, nrows=2)

axs = axs.flatten()

noms_axes = ['ouest', 'sud', 'est', 'horizontal', 'nord', '']

def formatage_axes():
    for N, ax in enumerate(axs):
        ax.set_title(noms_axes[N],y=0.98)
        ax.set_aspect('equal')
        ax.set_xlim(-largeur/2, largeur/2)
        ax.set_ylim(0, hauteur)
        ax.set_xticks([])
        ax.set_yticks([])
formatage_axes()

def lignes(D, H, a):
    """
    Parameters
    ----------
    D : TYPE : float
        DESCRIPTION : déclinaison du mur (degrés)
    H : TYPE : float
        DESCRIPTION. angle horaire (degrés)
    a : (float) hauteur du style droit.
    Returns
    -------
    renvoie les coordonnées (x, y) de la ligne horaire.
    """
    D_rad = np.radians(D)
    H_rad = np.radians(H)
    Q = np.sin(D_rad) * np.sin(H_rad) + np.sin(lat_rad) * np.cos(D_rad) * np.cos(H_rad) - np.cos(lat_rad) * np.cos(D_rad) * np.tan(decl)
    Nx = np.cos(D_rad) * np.sin(H_rad) - np.sin(lat_rad) * np.sin(D_rad) * np.cos(H_rad) + np.cos(lat_rad) * np.sin(D_rad) * np.tan(decl)
    Ny = - np.cos(lat_rad) * np.cos(H_rad) - np.sin(lat_rad) * np.tan(decl)
    x = a * Nx / Q
    y = a * Ny / Q
    return x, y

def declim(H_rad):
    declimite = np.arctan2(-np.cos(H_rad), np.tan(lat_rad))
    return declimite

def azimut(latitude, decl, H, deg = False):
    """
    latitude : float
    latitude du lieu (radians).
    H (float) : angle horaire du Soleil (radians)
    -----------------
    Renvoie l'azimut du Soleil compté depuis le sud (radians par défaut,
    en degrés si deg=True)
    """
    A = np.arctan2(np.sin(H), (np.sin(latitude) * np.cos(H) - np.cos(latitude) * np.tan(decl)))
    return A

def altitude(latitude, decl, H):
    """
    latitude : float
    latitude du lieu (radians).
    H (float) : angle horaire du Soleil (radians)
    ------------------
    Renvoie la hauteur du Soleil en radians, par défaut, degrés si deg = True.
    """
    sinh = np.sin(latitude)*np.sin(decl) + np.cos(latitude)*np.cos(decl)*np.cos(H)
    h = np.arcsin(sinh)
    return h

#  -------------------------CADRAN SUD  ----------------------------------
# a = 1.4
xmin, xmax = -largeur/2, largeur/2
ymax = 2
ymin = ymax-hauteur
#  centre du cadran (pied style polaire):
ax = axs[1]
ax.set_ylim(ymin, ymax)
ax.text(0.01, 0.01, f"style droit {a} cm", transform = ax.transAxes, c='indigo',
        ha='left')
#  point C pied du style polaire :
xC, yC = 0, a*np.tan(lat_rad)
# ax.scatter(xC, yC, s=10, c='white', ec='indigo', lw=0.5)
# point P : pied du style droit :
xP = 0
yP = 0
ax.scatter(xP, yP, s=10, c='white', ec='indigo', lw=0.5)
#  silhouette du style rectangulaire :
ax.plot([xC, xC + a, xP, xC],[yC, yP, yP, yC], ls='--', c='gray', lw=0.5)

#  tracé des lignes horaires :
for heure in np.arange(8, 17):
    H = (heure-12)*15
    H_rad = np.radians(H)
    # print(np.degrees(declim(H_rad)))
    declimite = max(declim(H_rad), -eps)
    decl = np.arange(declimite, eps, 0.001)
    x, y = lignes(0, H, a)
    ax.plot(x,y,**prop_lignesH)
    if len(x)>0 and x[-1] >= 0.95*xmin and x[-1] <= 0.95*xmax:
        ax.text(x[-1], 1.03*y[-1], heure, va='top', ha='center')

for heure in np.arange(8.5, 16):# demi-heures
    H = (heure-12)*15
    H_rad = np.radians(H)
    declimite = max(declim(H_rad), -eps)
    decl = np.arange(declimite, eps, 0.001)
    x, y = lignes(0, H, a)
    ax.plot(x,y,**prop_lignesdemies)

for heure in [7,17]:
    H = (heure-12)*15
    H_rad = np.radians(H)
    decl = np.arange(-eps, 0, 0.001)
    x, y = lignes(0, H, a)
    ax.plot(x,y, **prop_lignesH)
for heure in [6.5, 7.5, 16.5, 17.5]:# demi-heures centrales
    H = (heure-12)*15
    H_rad = np.radians(H)
    decl = np.arange(-eps, 0.1, 0.001)
    x, y = lignes(0, H, a)
    ax.plot(x,y, **prop_lignesdemies)

#  lignes 6h et 18 h :
ax.plot([xmin, xmin+0.15*largeur],[yC,yC], **prop_lignesH)
ax.text(xmin+0.16*largeur,yC, '6', va='center')
ax.plot([xmax, xmax-0.15*largeur],[yC,yC], **prop_lignesH)
ax.text(xmax-0.16*largeur,yC, '18', va='center',
        ha='right')

#  équinoxiale :
y_eq = -a*np.tan(colat)
ax.plot([xmin, xmax], [y_eq, y_eq], **prop_arcsdiurnes, zorder=0)

#  arc diurne été :
H = np.arange(-60, 60, 1)
decl = eps
x, y = lignes(0, H, a)
ax.plot(x,y, **prop_arcsdiurnes)
# arc diurne hiver :
decl = -eps
x, y = lignes(0, H, a)
ax.plot(x,y, **prop_arcsdiurnes)

#  --------------------cadran EST ----------------------------------
# a = 1.4
D = -90
#  centre du cadran (pied style polaire):
ax = axs[2]
ax.text(0.01, 0.01, f"style droit {a} cm", transform = ax.transAxes, c='indigo',
        ha='left')
xC, yC = -largeur * 1/3, hauteur *7/8
ax.scatter(xC, yC, s=10, c='white', ec='indigo', lw=0.5)
xmin = - largeur / 2
xmax = largeur / 2
# équinoxiale :
ax.plot([xC, xmax],[yC, -(xmax-xC)*np.tan(colat) + yC],
        **prop_arcsdiurnes)

# arc diurne été :
decl = eps
X, Y = [], []
for heure in np.arange(heuremin, 12, 0.1):
    H_rad = np.radians((heure-12)*15)
    r = - a / np.tan(H_rad)
    R = - a * np.tan(decl) / np.sin(H_rad)
    x = xC + r * np.cos(colat) + R * np.cos(np.pi - lat_rad)
    X.append(x)
    y = yC - r * np.sin(colat) - R * np.sin(np.pi - lat_rad)
    Y.append(y)
ax.plot(X, Y, **prop_arcsdiurnes)

# arc diurne hiver :
decl = -eps
X, Y = [], []
heure_lever_hiver = 12-np.arccos( - np.tan(-eps)*np.tan(lat_rad))/rad_15
for heure in np.arange(heure_lever_hiver, 12, 0.1):
    H_rad = np.radians((heure-12)*15)
    declimite = max(declim(H_rad), -eps)
    r = - a / np.tan(H_rad)
    R = - a * np.tan(decl) / np.sin(H_rad)
    x = xC + r * np.cos(colat) + R * np.cos(np.pi - lat_rad)
    X.append(x)
    y = yC - r * np.sin(colat) - R * np.sin(np.pi - lat_rad)
    Y.append(y)
ax.plot(X, Y, **prop_arcsdiurnes)

#  lignes horaires : --------------------------------------------
for heure in np.arange(heuremin, 12, 1):
    H_rad = np.radians((heure-12)*15)
    r = - a / np.tan(H_rad)
    declimite = declim(H_rad)
    declimite = max(declimite, -eps)
    decl = np.arange(eps, declimite,-0.001)
    R = - a * np.tan(decl) / np.sin(H_rad)
    x = xC + r * np.cos(colat) + R * np.cos(np.pi - lat_rad)
    y = yC - r * np.sin(colat) - R * np.sin(np.pi - lat_rad)
    ax.plot(x, y, **prop_lignesH)
    if len(x)>0 :
        ax.text(0.999*x[-1], 1.005*y[-1], heure, ha='left', va='bottom')

for heure in np.arange(heuremin+0.5, 12, 1):# demi-heures
    H_rad = np.radians((heure-12)*15)
    r = - a / np.tan(H_rad)
    declimite = declim(H_rad)
    declimite = max(declimite,-eps)
    decl = np.arange(eps, declimite,-0.001)
    R = - a * np.tan(decl) / np.sin(H_rad)
    x = xC + r * np.cos(colat) + R * np.cos(np.pi - lat_rad)
    y = yC - r * np.sin(colat) - R * np.sin(np.pi - lat_rad)
    ax.plot(x, y, **prop_lignesdemies)

# ----------------------- cadran OUEST ----------------------------------
# a = 1.4
#  centre du cadran (pied style polaire):
ax = axs[0]
ax.text(0.99, 0.01, f"style droit {a} cm", transform = ax.transAxes, c='indigo',
        ha='right')
xC, yC = largeur * 1/3, hauteur *7/8
ax.scatter(xC, yC, s=10, c='white', ec='indigo', lw=0.5)
xmin = - largeur / 2
xmax = largeur / 2
# équinoxiale :
ax.plot([xmin, xC],[(xmin-xC)*np.tan(colat) + yC, yC],**prop_arcsdiurnes)

# arc diurne été :
decl = eps
X, Y = [], []
for heure in np.arange(12.1, heuremax, 0.1):
    H_rad = np.radians((heure-12)*15)
    r = a / np.tan(H_rad)
    R = a * np.tan(decl) / np.sin(H_rad)
    x = xC - r * np.cos(colat) - R * np.cos(np.pi - lat_rad)
    X.append(x)
    y = yC - r * np.sin(colat) - R * np.sin(np.pi - lat_rad)
    Y.append(y)
ax.plot(X, Y, **prop_arcsdiurnes)

# arc diurne hiver :
decl = -eps
X, Y = [], []
heure_coucher_hiver = 12+np.arccos( - np.tan(-eps)*np.tan(lat_rad))/rad_15
for heure in np.arange(12.1, heure_coucher_hiver, 0.1):
    H_rad = np.radians((heure-12)*15)
    r = a / np.tan(H_rad)
    R = a * np.tan(decl) / np.sin(H_rad)
    x = xC - r * np.cos(colat) - R * np.cos(np.pi - lat_rad)
    X.append(x)
    y = yC - r * np.sin(colat) - R * np.sin(np.pi - lat_rad)
    Y.append(y)
ax.plot(X, Y, **prop_arcsdiurnes)

#  lignes horaires : ----------------------------------------

for heure in np.arange(13, heuremax+1, 1):
    H_rad = np.radians((heure-12)*15)
    r = a / np.tan(H_rad)
    declimite = declim(H_rad)
    declimite = max(declimite, -eps)
    decl = np.arange(eps, declimite, -0.001)
    R = a * np.tan(decl) / np.sin(H_rad)
    x = xC - r * np.cos(colat) - R * np.cos(np.pi - lat_rad)
    y = yC - r * np.sin(colat) - R * np.sin(np.pi - lat_rad)
    ax.plot(x, y, **prop_lignesH)
    if len(x)>0 :
        ax.text(1.01*x[-1], 1.005*y[-1], heure, ha='right', va='bottom')

for heure in np.arange(12.5, heuremax, 1):# demi-heures
    H_rad = np.radians((heure-12)*15)
    r = a / np.tan(H_rad)
    declimite = declim(H_rad)
    declimite = max(declimite,-eps)
    decl = np.arange(eps, declimite, -0.001)
    R = a * np.tan(decl) / np.sin(H_rad)
    x = xC - r * np.cos(colat) - R * np.cos(np.pi - lat_rad)
    y = yC - r * np.sin(colat) - R * np.sin(np.pi - lat_rad)
    ax.plot(x, y, **prop_lignesdemies)

#  ---------------------------CADRAN NORD -----------------------
ax = axs[4]
# a = 1.4
D = 180
ax.text(0.01, 0.01, f"style droit {a} cm", transform = ax.transAxes, c='indigo',
        ha='left')
xmin, xmax = -largeur/2, largeur/2
ymax = 2
ymin = ymax-hauteur
# point P : pied du style droit :
xP = 0
yP = 0
# C centre du cadran (pied style polaire):
xC, yC = 0, yP - a*np.tan(lat_rad)
ax.set_ylim(ymin, ymax)
# ax.scatter(xC, yC, s=10, c='white', ec='indigo', lw=0.5)
ax.scatter(xP, yP, s=10, c='white', ec='indigo', lw=0.5)
#  silhouette du style rectangulaire :
ax.plot([xP, xP + a, xC, xP],[yP, yP, yC, yP], ls='--', c='gray', lw=0.5)

#  tracé des lignes horaires :
heures = [heuremin, heuremin+1, heuremax-1,heuremax-2, heuremax]
decl = np.arange(0, eps, 0.001)
for heure in heures:
    H = (heure-12)*15
    x, y = lignes(D, H, a)
    ax.plot(x,y,**prop_lignesH)
    if len(x)>0 and x[-1] >= xmin and x[-1] <= xmax:
        ax.text(0.9*x[-1], 1.03*y[-1], heure, va='top', ha='center')
    x = -x
    y = -(y- yC) + yC
    if heure < 12:# trace les heures opposées (12 + heure)
        heure = heure + 12
    else:
        heure = heure -12
    ax.plot(x,y,**prop_lignesH)
    if len(x)>0 and x[-1] <= xmax:
       ax.text(0.9*x[-1], 0.90*y[-1], heure, va='top', ha='center')

#  demies :
heures = [heuremin+0.5, 5.5, 18.5, 19.5]
decl = np.arange(0, eps, 0.001)
for heure in heures:
    H = (heure-12)*15
    x, y = lignes(D, H, a)
    ax.plot(x,y,**prop_lignesdemies)
    if heure in [heuremin+.5,5.5, 18.5,19.5, heuremax-0.5]:
        x = -x
        y = -(y- yC) + yC
        if heure < 12:
            heure = heure + 12
        else:
            heure = heure -12
        ax.plot(x,y,**prop_lignesdemies)

#  lignes 6h et 18h :
ax.plot([xmin, xmin+0.15*largeur],[yC,yC], **prop_lignesH)
ax.text(xmin+0.16*largeur,yC, '18', va='center')
ax.plot([xmax, xmax-0.15*largeur],[yC,yC], **prop_lignesH)
ax.text(xmax-0.16*largeur,yC, '6', va='center',
        ha='right')


# ------------------ CADRAN HORIZONTAL -----------------------------------
# a = 1.4
ax = axs[3]
# ax.text(0.01, 0.99, f"style droit {a} cm", transform = ax.transAxes, c='indigo',
#         fontsize=5, ha='left', va='top')
xmin, xmax = -largeur/2, largeur/2
ymin = -3
ymax = ymin + hauteur
ax.set_xlim(xmin, xmax)
ax.set_ylim(ymin, ymax)
xP, yP = 0, 0 # coordonnées du pied du style droit
xC = 0
yC = yP - a/np.tan(lat_rad) # centre du cadran (pied style polaire)
ax.scatter(xP, yP, s=10, c='white', ec='indigo', lw=0.5)
# ax.scatter(xC, yC, s=10, c='white', ec='indigo', lw=0.5)
#  silhouette du style rectangulaire :
ax.plot([xP, xP + a, xC, xP],[yP, yP, yC, yP], ls='--', c='gray', lw=0.5)

#  ligne midi :
ax.plot([0,0],[ymax, ymax-4], **prop_lignesH)
decl = np.arange(-eps, eps, 0.01)

for heure in np.arange(heuremin,12):
    H = (heure-12)*15
    H_rad = np.radians(H)
    angle = np.pi/2 + np.arctan(- np.sin(lat_rad) * np.tan(H_rad))
    # x = a * np.sin(H) / (np.cos(lat_rad)*np.cos(H)+np.sin(lat_rad)*np.tan(decl))
    # y = a * (np.sin(lat_rad) * np.cos(H)-np.cos(lat_rad)*np.tan(decl)) / (np.cos(lat_rad)*np.cos(H) + np.sin(lat_rad)*np.tan(decl))
    if heure <11:
        x0 = -1
    else:x0=-0.5
    x = np.arange(xmin, x0, 0.001)
    y = x*np.tan(angle) + yC
    ax.plot(x,y,**prop_lignesH)
    if len(x)>0 and x[0] >= xmin and x[0] <= xmax:
        ax.text(0.8*x[-1], 1.03*y[-1], heure, va='center',
                ha='center')
#  demies :
for heure in np.arange(heuremin+0.5,12):
    H = (heure-12)*15
    H_rad = np.radians(H)
    angle = np.pi/2 + np.arctan(- np.sin(lat_rad) * np.tan(H_rad))
    if heure <11.0:
        x0 = -1
    else:x0=-0.5
    # x = np.arange(x0, xmax, 0.001)
    x = np.arange(xmin,x0, 0.001)
    y = x*np.tan(angle) + yC
    ax.plot(x,y,**prop_lignesdemies)

for heure in np.arange(13, heuremax+1):
    H = (heure-12)*15
    H_rad = np.radians(H)
    angle = np.pi/2 + np.arctan(- np.sin(lat_rad) * np.tan(H_rad))
    if heure >13:
        x0 = 1
    else:x0=0.5
    x = np.arange(x0, xmax, 0.001)
    y = x*np.tan(angle) + yC
    ax.plot(x,y,**prop_lignesH)
    if len(x)>0 and x[0] >= xmin and x[0] <= xmax:
        ax.text(0.8*x[0], 1.03*y[0], heure, va='center',
                ha='center')

#  demies :
for heure in np.arange(12.5, heuremax+0.5):
    H = (heure-12)*15
    H_rad = np.radians(H)
    angle = np.pi/2 + np.arctan(- np.sin(lat_rad) * np.tan(H_rad))
    if heure >13:
        x0 = 1
    else:x0=0.5
    x = np.arange(x0, xmax, 0.001)
    y = x*np.tan(angle) + yC
    ax.plot(x,y,**prop_lignesdemies)

#  équinoxiale :
y_eq = a/np.tan(colat)
ax.plot([xmin,xmax],[y_eq, y_eq], **prop_arcsdiurnes)


#  --------------------------------------------------------------------
inch = 2.54 # conversion des cm en pouces
fig_width = (largeur*3+marge*4) / inch# largeur du document entier
fig_height = (hauteur*2+marge*3) / inch# hauteur du document entier
marge_G = marge / inch# largeur de la marge gauche
largeur_axe = (largeur*3 + marge*4) / inch
hauteur_axe = (hauteur*2 + marge*3) / inch
# marge_D=fig_width-marge_G-largeur_axe# calcul marge droite
marge_D = marge / inch# calcul marge droite
marge_B=marge / inch # marge basse
# marge_H=fig_height-marge_B-hauteur_axe# marge haute
marge_H = marge / inch# marge haute

#  ajustement des marges et de la taille du cadre :
plt.subplots_adjust(
    left=marge_G / fig_width,
    right=1-marge_D/fig_width,
    bottom= marge_B / fig_height,
    top= 1 - marge_H / fig_height,
    wspace= marge / inch / largeur_axe,
    hspace= 3*marge / inch / hauteur_axe
)

fig.set_size_inches(fig_width, fig_height)# dimensions finales de la figure

# INSTRUCTIONS -------------------------------
ax = axs[5]
texte = "Coller ces figures sur les 5 faces apparentes\nd'un cube.\nLes styles droits sont perpendiculaires à la face\ndes cubes. Leurs points d'implantation sont\nrepérés par des cercles.\nLes styles triangulaires sont dessinés rabattus\nde 90° sur le plan."
ax.text(0.02,0.98, texte,ha='left',va='top',
        transform = ax.transAxes)

ax.text(0.5, 0.02, f"lat. {latitude}°", transform = ax.transAxes,
        ha='center')

fig.savefig(nomFichier + ".pdf")# sauvegarde du fichier pdf
fig.savefig(nomFichier + ".png" , dpi=250)# sauvegarde du fichier png
